package de.fhmuenster.masterthesis.formatting2

import com.google.inject.Inject
import de.fhmuenster.masterthesis.services.TestgeneratorDSLGrammarAccess
import de.fhmuenster.masterthesis.testgeneratorDSL.BPMNDiagram
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanVariable
import de.fhmuenster.masterthesis.testgeneratorDSL.Declaration
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceMock
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalTopic
import de.fhmuenster.masterthesis.testgeneratorDSL.Flow
import de.fhmuenster.masterthesis.testgeneratorDSL.FlowElement
import de.fhmuenster.masterthesis.testgeneratorDSL.IntVariable
import de.fhmuenster.masterthesis.testgeneratorDSL.Loop
import de.fhmuenster.masterthesis.testgeneratorDSL.Mock
import de.fhmuenster.masterthesis.testgeneratorDSL.MockingRule
import de.fhmuenster.masterthesis.testgeneratorDSL.Model
import de.fhmuenster.masterthesis.testgeneratorDSL.StringVariable
import de.fhmuenster.masterthesis.testgeneratorDSL.Test
import de.fhmuenster.masterthesis.testgeneratorDSL.Variable
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableDeclarations
import java.util.List
import org.eclipse.xtext.formatting2.AbstractFormatter2
import org.eclipse.xtext.formatting2.IFormattableDocument

class TestgeneratorDSLFormatter extends AbstractFormatter2 {
	
	@Inject extension TestgeneratorDSLGrammarAccess

	def dispatch void format(Model model, extension IFormattableDocument document) {
		
		model.regionFor.keyword("with").prepend[newLine; oneSpace]
		model.regionFor.keyword(";").prepend[noSpace]
		model.append[newLine]
		
		var List<Declaration> declarationsList = model.getDeclarations();
		var decCount = 0;
		var maxDecCount = declarationsList.size;
		for (Declaration declarations : declarationsList) {
			declarations.format;
			
			if(decCount < (maxDecCount - 1)) {
				var FormattingType formattingType = getFormattingType(declarations);
				var FormattingType nextFormattingType = getFormattingType(declarationsList.get(decCount + 1));
				
				if(shouldSeparateDouble(formattingType, nextFormattingType)) {
					declarations.append[newLines = 2]				
				} else {
					declarations.append[newLine]				
				}
			}
			
			decCount++;
		}
	}

	def dispatch void format(BPMNDiagram diagram, extension IFormattableDocument document) {
		diagram.regionFor.keyword('with').prepend[newLine]
	}
	
	def dispatch void format(Loop loop, extension IFormattableDocument document) {
		loop.regionFor.keyword('starts').prepend[newLine]
		loop.regionFor.keyword('follows').prepend[newLine]
		loop.regionFor.keyword('ends').prepend[newLine]
		loop.regionFor.keyword('has').prepend[newLine]
	}
	
	def dispatch void format(Flow flow, extension IFormattableDocument document) {
		flow.regionFor.keyword('with').prepend[newLine]
		flow.regionFor.keyword('without').prepend[newLine]
	}
	
	def dispatch void format(FlowElement flowElement, extension IFormattableDocument document) {
		//flowElement.regionFor.keyword('with').prepend[newLine; oneSpace]
		flowElement.regionFor.keyword(":").prepend[noSpace]
		flowElement.regionFor.keyword(";").prepend[noSpace]
	}
	
	def dispatch void format(ExternalTopic externalTopic, extension IFormattableDocument document) {
		externalTopic.regionFor.keyword(";").prepend[noSpace]
	}
	
	def dispatch void format(StringVariable stringVariable, extension IFormattableDocument document) {
	}
	
	def dispatch void format(IntVariable integerVariable, extension IFormattableDocument document) {
	}
	
	def dispatch void format(BooleanVariable booleanVariable, extension IFormattableDocument document) {
	}
	
	def dispatch void format(ExternalServiceMock mock, extension IFormattableDocument document) {
		mock.regionFor.keyword('with').prepend[newLine]

		for(MockingRule rule : mock.getMockingRules()) {
			rule.format;
			rule.prepend[newLine];
		}
	}
	
	def dispatch void format(MockingRule rule, extension IFormattableDocument document) {
		rule.regionFor.keyword('with').prepend[oneSpace]
	}
	
	def dispatch void format(Test test, extension IFormattableDocument document) {
		test.regionFor.keyword('with').prepend[newLine]
		
		for (VariableDeclarations declarations : test.getDeclarations()) {
			declarations.format;
			declarations.prepend[newLine]
		}
		
		if(test.getEndCheck() != null) {
			test.getEndCheck().prepend[newLine];
		}
	}
	
	def dispatch void format(VariableDeclarations variableDeclarations, extension IFormattableDocument document) {
		variableDeclarations.regionFor.keyword('with').prepend[oneSpace]
	}
	
	def FormattingType getFormattingType(Declaration declarations) {
		if(declarations instanceof BPMNDiagram) {
			return FormattingType.PROCESS;
		}
		
		if(declarations instanceof Loop) {
			return FormattingType.LOOP;
		}
		
		if(declarations instanceof Flow) {
			return FormattingType.FLOW;
		}
		
		if(declarations instanceof FlowElement) {
			return FormattingType.FLOW_ELEMENT;
		}
		
		if(declarations instanceof Variable) {
			return FormattingType.VARIABLE;
		}
		
		if(declarations instanceof Mock) {
			return FormattingType.MOCK;
		}
		
		if(declarations instanceof Test) {
			return FormattingType.TEST;
		}
		
		if(declarations instanceof ExternalTopic) {
			return FormattingType.EXTERNAL_TOPIC;
		}
		
		return null;
	}
	
	def boolean shouldSeparateDouble(FormattingType first, FormattingType second) {
		if(!first.equals(second)) {
			return true;
		}
		
		if(first.equals(FormattingType.TEST) || first.equals(FormattingType.MOCK)) {
			return true;
		}
		
		return false;
	}
}
